﻿Type=Class
Version=3.5
@EndOfDesignText@
'Class module
'version 1.24
'See this page for the list of constants:
'http://developer.android.com/intl/fr/reference/android/hardware/Camera.Parameters.html
'Note that you should use the constant values instead of the names.
'Changes by Jim Brown (v1.21)
'  Added support for scene modes, white balance, focus, zoom, SetPreviewSize.
'  Added helper parameters for GetSupported... functions.
'  Changed certain subs to set/get types. E.g, camEx.JpegQuality=90 instead of camEx.SetJpegQuality(90)
'Changes by Aeropic (v1.22)
'  Added support for scene modes, white balance, focus, zoom, SetPreviewSize.
'changes by Aeropic (V1.23)
'  Added support for focusareas (credits Agraham)
'changes by MaFu (V1.24)
'  Added GetRotation and GetSupportedPreviewSizes
'
Sub Class_Globals
	Private nativeCam As Object
	Private cam As Camera
	Private r As Reflector
	Private target As Object
	Private event As String
	Public Front As Boolean
	Type CameraInfoAndId (CameraInfo As Object, Id As Int)
	Type CameraSize (Width As Int, Height As Int)
	Private parameters As Object
	Private shoot As Boolean
	Private pFocus As Panel
	Private cFocus As Canvas
End Sub

Public Sub Initialize (Panel1 As Panel, FrontCamera As Boolean, TargetModule As Object, EventName As String, panelfocus As Panel, canfocus As Canvas)
	target = TargetModule
	event = EventName
	Front = FrontCamera
	'pFocus.Initialize("pfocus")
	'cFocus.Initialize(pFocus)
	pFocus = panelfocus
	cFocus = canfocus
	
	Dim id As Int
	id = FindCamera(Front).id
	If id = -1 Then
		Front = Not(Front) 'try different camera
		id = FindCamera(Front).id
		If id = -1 Then
			ToastMessageShow("No camera found.", True)
			Return
		End If
	End If
	cam.Initialize2(Panel1, "camera", id)
End Sub

Private Sub FindCamera (frontCamera As Boolean) As CameraInfoAndId
	Dim ci As CameraInfoAndId
	Dim cameraInfo As Object
	Dim cameraValue As Int
	If frontCamera Then cameraValue = 1 Else cameraValue = 0
	cameraInfo = r.CreateObject("android.hardware.Camera$CameraInfo")
	Dim numberOfCameras As Int = r.RunStaticMethod("android.hardware.Camera", "getNumberOfCameras", Null, Null)
	For i = 0 To numberOfCameras - 1
		r.RunStaticMethod("android.hardware.Camera", "getCameraInfo", Array As Object(i, cameraInfo), _
			Array As String("java.lang.int", "android.hardware.Camera$CameraInfo"))
		r.target = cameraInfo
		If r.GetField("facing") = cameraValue Then
			ci.cameraInfo = r.target
			ci.Id = i
			Return ci
		End If
	Next
	ci.id = -1
	Return ci
End Sub

Private Sub SetDisplayOrientation
	r.target = r.GetActivity
	r.target = r.RunMethod("getWindowManager")
	r.target = r.RunMethod("getDefaultDisplay")
	r.target = r.RunMethod("getRotation")
	Dim previewResult, result, degrees As Int = r.target * 90
	Dim ci As CameraInfoAndId = FindCamera(Front)
	r.target = ci.CameraInfo
	Dim orientation As Int = r.GetField("orientation")
	If Front Then
		previewResult = (orientation + degrees) Mod 360
		result = previewResult
		previewResult = (360 - previewResult) Mod 360
	Else
		previewResult = (orientation - degrees + 360) Mod 360
		result = previewResult
		Log(previewResult)
	End If
	r.target = nativeCam
	r.RunMethod2("setDisplayOrientation", previewResult, "java.lang.int")
	r.target = parameters
	r.RunMethod2("setRotation", result, "java.lang.int")
	CommitParameters
End Sub

Private Sub Camera_Ready (Success As Boolean)
	If Success Then
		r.target = cam
		nativeCam = r.GetField("camera")
		r.target = nativeCam
		parameters = r.RunMethod("getParameters")
		SetDisplayOrientation
	Else
		Log("success = false, " & LastException)
	End If
	CallSub2(target, event & "_ready", Success)
End Sub

Sub Camera_Preview (Data() As Byte)
	If SubExists(target, event & "_preview") Then
		CallSub2(target, event & "_preview", Data)
	End If
End Sub

Public Sub TakePicture
	pFocus.Visible = False
	cam.TakePicture
End Sub

Private Sub Camera_PictureTaken (Data() As Byte)
	CallSub2(target, event & "_PictureTaken", Data)
End Sub

Public Sub StartPreview
	cam.StartPreview
End Sub

Public Sub StopPreview
	cam.StopPreview
End Sub

Public Sub Release
	cam.Release
End Sub

'Saves the data received from PictureTaken event
Public Sub SavePictureToFile(Data() As Byte, Dir As String, FileName As String)
	Dim out As OutputStream = File.OpenOutput(Dir, FileName, False)
	out.WriteBytes(Data, 0, Data.Length)
	out.Close
End Sub

Public Sub GetParameter(Key As String) As String
	r.target = parameters
	Return r.RunMethod2("get", Key, "java.lang.String")
End Sub

Public Sub SetParameter(Key As String, Value As String)
	r.target = parameters
	r.RunMethod3("set", Key, "java.lang.String", Value, "java.lang.String")
End Sub

Public Sub CommitParameters
	Try
		r.target = nativeCam
		r.RunMethod4("setParameters", Array As Object(parameters), Array As String("android.hardware.Camera$Parameters"))
	Catch
		Log("Error setting parameters.")
		ToastMessageShow("Error setting parameters.", True)
		Log(LastException)
	End Try
End Sub

'Gets and sets the color effect. getColorEffect returns empty string (null) if not supported.
'See: GetSupportedColorEffects
Public Sub getColorEffect As String
	Return GetParameter("effect")
End Sub

Public Sub setColorEffect(Effect As String)
	SetParameter("effect", Effect)
End Sub

'Retuns a list of supported color effects. List will be be uninitialized (null) if not supported.
'Options: "none" , "mono" , "negative" , "solarize" , "sepia" , "posterize" , "whiteboard" , "blackboard" , "aqua"
'Other:   "aqua" , "emboss" , "sketch" , "neon" , "vintage-warm" , "vintage-cold" , "washed"
Public Sub GetSupportedColorEffects As List
	r.target = parameters
	Return r.RunMethod("getSupportedColorEffects")
End Sub

'Gets and sets the scene modes. getSceneMode returns empty string (null) if not supported.
'See: GetSupportedSceneModes
Public Sub getSceneMode As String
	Return GetParameter("mode")
End Sub

Public Sub setSceneMode(Mode As String)
	SetParameter("mode", Mode)
End Sub

'Retuns a list of supported scene modes. List will be be uninitialized (null) if not supported.
'Options: "auto" , "action" , "portrait" , "landcape" , "night" , "night-portrait" , "theatre" , "beach" , "snow"
'         "sunset" , "steadyphoto" , "fireworks" , "sports" , "party" , "candlelight" , "barcode"
'Other:   "asd"
Public Sub GetSupportedSceneModes As List
	r.target = parameters
	Return r.RunMethod("getSupportedSceneModes")
End Sub

'Returns a CameraSize type containing the width and height of the picture size
Public Sub GetPictureSize As CameraSize
	r.target = parameters
	r.target = r.RunMethod("getPictureSize")
	Dim cs As CameraSize
	cs.Width = r.GetField("width")
	cs.Height = r.GetField("height")
	Return cs
End Sub

'Set the picture size.
Public Sub SetPictureSize(Width As Int, Height As Int)
	r.target = parameters
	r.RunMethod3("setPictureSize", Width, "java.lang.int", Height, "java.lang.int")
End Sub

'Returns an array of CameraSize types containing the available picture sizes.
Public Sub GetSupportedPicturesSizes As CameraSize()
	r.target = parameters
	Dim list1 As List = r.RunMethod("getSupportedPictureSizes")
	Dim cs(list1.Size) As CameraSize
	For i = 0 To list1.Size - 1
		r.target = list1.Get(i)
		cs(i).Width = r.GetField("width")
		cs(i).Height = r.GetField("height")
	Next
	Return cs
End Sub

'Gets or sets the JPeg quality - Range = 1 (Worst) to 100 (Best)
Public Sub getJpegQuality() As Int
	r.target = parameters
	Return r.RunMethod("getJpegQuality")
End Sub

Public Sub setJpegQuality(Quality As Int)
	If Quality<1 Then Quality=1
	If Quality>100 Then Quality=100
	r.target = parameters
	r.RunMethod2("setJpegQuality", Quality, "java.lang.int")
End Sub

'Gets or sets the current zoom for snapshots.
'Call isZoomSupported() to evaluate if the device supports zooming.
'Call GetMaxZoom() to determine the maximum zoom amount available.
Public Sub getZoom() As Int
	If isZoomSupported Then
		r.target = parameters
		Return r.RunMethod("getZoom")
	Else
		'Log("Zoom not supported.")
		Return 0
	End If
End Sub

Public Sub setZoom(Value As Int)
	Value=Max(Value,GetMaxZoom)
	If isZoomSupported Then
		r.target = parameters
		r.RunMethod2("setZoom", Value, "java.lang.int")
	Else
		Log("Zoom not supported.")
	End If
End Sub

'Returns the the maximum zoom range supported by the device.
'A value of 0 is returned if zooming is not available.
'NOTE: Call this again after any SetPreviewSize() changes.
Public Sub GetMaxZoom() As Int
	If isZoomSupported Then
		r.target = parameters
		Return r.RunMethod("getMaxZoom")
	Else
		'Log("Zoom not supported.")
		Return 0
	End If
End Sub

'Returns True if zooming is supported.
Public Sub isZoomSupported() As Boolean
	r.target = parameters
	Return r.RunMethod("isZoomSupported")
End Sub

'Gets or sets flash mode. getFlashMode returns empty string (null) if featue not supported.
'See: GetSupportedFlashModes
Public Sub getFlashMode As String
	r.target = parameters
	Return r.RunMethod("getFlashMode")
End Sub

Public Sub setFlashMode(Mode As String)
	r.target = parameters
	r.RunMethod2("setFlashMode", Mode, "java.lang.String")
End Sub

'Returns a list of supported flash modes. List is not initialized (null) if feature not supported.
'Options: "off" , "auto" , "on" , "red-eye" , "torch"
Public Sub GetSupportedFlashModes As List
	r.target = parameters
	Return r.RunMethod("getSupportedFlashModes")
End Sub

'Gets or sets the white balance. getWhiteBalance returns empty string (null) if not supported.
'See: GetSupportedWhiteBalance
Public Sub getWhiteBalance As String
	r.target = parameters
	Return r.RunMethod("getWhiteBalance")
End Sub

Public Sub setWhiteBalance (Mode As String)
   r.target = parameters
   r.RunMethod2("setWhiteBalance", Mode, "java.lang.String")
End Sub

'Returns a list of supported white balance options. See getWhiteBalance/setWhiteBalance
'Options: "auto" , "incandescent" , "fluorescent"" , "warm-fluorescent" , "daylight"
'         "cloudy-daylight" , "twighlight" , "shade"
Public Sub GetSupportedWhiteBalance  As List
	r.target = parameters
	Return r.RunMethod("getSupportedWhiteBalance")
End Sub

'Returns a CameraSize type containing the Width and Height preview size.
Public Sub GetPreviewSize As CameraSize
	r.target = parameters
	r.target = r.RunMethod("getPreviewSize")
	Dim cs As CameraSize
	cs.Width = r.GetField("width")
	cs.Height = r.GetField("height")
	Return cs
End Sub

'Sets the preview size. This should be called before startPreview().
'The sizes are based on camera orientation (before the image is rotated by the display orientation).
Public Sub SetPreviewSize(Width As Int, Height As Int)
	r.target = parameters
	r.RunMethod3("setPreviewSize", Width, "java.lang.int", Height, "java.lang.int")
End Sub

'Converts a preview image formatted in YUV format to JPEG.
'Note that you should not save every preview image as it will slow down the whole process.
Public Sub PreviewImageToJpeg(data() As Byte, quality As Int) As Byte()
	Dim size, previewFormat As Object
	r.target = parameters
	size = r.RunMethod("getPreviewSize")
	previewFormat = r.RunMethod("getPreviewFormat")
	r.target = size
	Dim width = r.GetField("width"), height = r.GetField("height") As Int
	Dim yuvImage As Object = r.CreateObject2("android.graphics.YuvImage", _
		Array As Object(data, previewFormat, width, height, Null), _
		Array As String("[B", "java.lang.int", "java.lang.int", "java.lang.int", "[I"))
	r.target = yuvImage
	Dim rect1 As Rect
	rect1.Initialize(0, 0, r.RunMethod("getWidth"), r.RunMethod("getHeight"))
	Dim out As OutputStream
	out.InitializeToBytesArray(100)
	r.RunMethod4("compressToJpeg", Array As Object(rect1, quality, out), _
		Array As String("android.graphics.Rect", "java.lang.int", "java.io.OutputStream"))
	Return out.ToBytesArray
End Sub

'Gets or sets the focus mode. getFocusMode will always return a string value.
'See: GetSupportedFocusModes    (also SetContinuousAutoFocus)
Public Sub getFocusMode As String
	r.target = parameters
	Return r.RunMethod("getFocusMode")
End Sub

Public Sub setFocusMode(Mode As String)
    r.target = parameters
    r.RunMethod2("setFocusMode", Mode, "java.lang.String")
End Sub

'Returns a list of supported focus modes. Always returns as least one element.
'See getFocusMode and setFocusMode
'Options: "auto" , "infinity" , "macro" , "fixed" , edof" , "continuous-picture" , "continuous-video"
Public Sub GetSupportedFocusModes As List
    r.target = parameters
    Return r.RunMethod("getSupportedFocusModes")
End Sub

'Sets continuous auto focus if available.
'Call this sub between startPreview() and stopPreview()
Public Sub SetContinuousAutoFocus
    Dim modes As List = GetSupportedFocusModes
    If modes.IndexOf("continuous-picture") > -1 Then
        setFocusMode("continuous-picture")
		Log("Continuous-picture")
    Else If modes.IndexOf("continuous-video") > -1 Then
        setFocusMode("continuous-video")
		Log("Continuous-video")
	Else If modes.IndexOf("continuous") > -1 Then
		setFocusMode("continuous")
		Log("Continuous")
    Else
        Log("Continuous focus mode is not available")
    End If
End Sub

'Returns an array of 3 floats for the current focus distances in meters.
'The 3 floats are Near, Optical, Far
Public Sub GetFocusDistances As Float()
	Dim F(3) As Float
	r.target = parameters
	r.RunMethod4("getFocusDistances", Array As Object(F), Array As String("[F"))
	Return F
End Sub


'Public Int getMaxNumFocusAreas ()  API >= 14
'Gets the maximum number of focus areas supported. This Is the maximum length of the List In setFocusAreas(List) AND getFocusAreas().
Public Sub GetMaxNumFocusAreas As Int
    r.target = parameters
    Return r.RunMethod("getMaxNumFocusAreas")
End Sub



'public List<Camera.Area> getFocusAreas ()  API >= 14
'Gets the current focus areas. Camera driver uses the areas to decide focus.
Public Sub GetFocusAreas As List
    r.target = parameters
    Return r.RunMethod("getFocusAreas")
End Sub
'Before using this API or setFocusAreas(List), apps should call getMaxNumFocusAreas() to know the maximum number of focus areas first.
'If the value is 0, focus area is not supported.
'Each focus area is a rectangle with specified weight. The direction is relative to the sensor orientation, that is, what the sensor sees.
'The direction is not affected by the rotation or mirroring of setDisplayOrientation(int).
'Coordinates of the rectangle range from -1000 to 1000. (-1000, -1000) is the upper left point. (1000, 1000) is the lower right point.
'The width and height of focus areas cannot be 0 or negative.

'The weight must range from 1 to 1000. The weight should be interpreted as a per-pixel weight - all pixels in the area have the specified weight.
'This means a small area with the same weight as a larger area will have less influence on the focusing than the larger area.
'Focus areas can partially overlap and the driver will add the weights in the overlap region.

'A special case of a null focus area list means the driver is free to select focus targets as it wants.
'For example, the driver may use more signals to select focus areas and change them dynamically.
'Apps can set the focus area list to null if they want the driver to completely control focusing.

'Focus areas are relative to the current field of view (getZoom()).
'No matter what the zoom level is, (-1000,-1000) represents the top of the currently visible camera frame.
'The focus area cannot be set to be outside the current field of view, even when using zoom.

'Focus area only has effect if the current focus mode is FOCUS_MODE_AUTO, FOCUS_MODE_MACRO, FOCUS_MODE_CONTINUOUS_VIDEO, or FOCUS_MODE_CONTINUOUS_PICTURE.


'public void setFocusAreas (List<Camera.Area> focusAreas) API >= 14
'Sets focus areas. See getFocusAreas() for documentation.
' use this to set only one focus area
Public Sub SetFocusArea(area As Rect, weight As Int )
   Dim r1, r2 As Reflector
   Dim params() As Object
   Dim types() As String
   params = Array As Object(area, weight)
   types = Array As String("android.graphics.Rect", "java.lang.int")
   r1.target = r1.CreateObject2("android.hardware.Camera$Area", params, types)
  ' Msgbox(r1.TypeName,"")

   Dim list1 As List
   list1.Initialize
   list1.Add(r1.target)
   r2.target = parameters
   r2.RunMethod4("setFocusAreas", Array As Object(list1), Array As String("java.util.List"))

   CommitParameters
End Sub

' Public final void cancelAutoFocus ()
'Cancels any auto-focus function In progress. 
'Whether OR Not auto-focus Is currently In progress, this function will Return the focus position To the default. 
'If the Camera does Not support auto-focus, this Is a no-op.
Public Sub cancelAutoFocus
	cam.cancelAutoFocus
End Sub


'Public Float getExposureCompensationStep ()
'Gets the exposure compensation Step.
'Applications can get EV by multiplying the exposure compensation index AND Step. Ex: If exposure compensation index Is -6 AND Step Is 0.333333333, EV Is -2.
Public Sub GetExposureCompensationStep As Float
    r.target = parameters
    Return r.RunMethod("getExposureCompensationStep")
End Sub

' Public Int getExposureCompensation ()
' Gets the current exposure compensation index.
' The range Is getMinExposureCompensation() To getMaxExposureCompensation(). 0 means exposure Is Not adjusted.
Public Sub GetExposureCompensation As Int
    r.target = parameters
    Return r.RunMethod("getExposureCompensation")
End Sub

'Public Int getMaxExposureCompensation ()
' Gets the maximum exposure compensation index.
'maximum exposure compensation index (>=0). If both this method AND getMinExposureCompensation() Return 0, exposure compensation Is Not supported.
Public Sub GetMaxExposureCompensation As Int
    r.target = parameters
    Return r.RunMethod("getMaxExposureCompensation")
End Sub

'Public Int getMinExposureCompensation ()
'Gets the minimum exposure compensation index.
'minimum exposure compensation index (<=0). If both this method AND getMaxExposureCompensation() Return 0, exposure compensation Is Not supported.
Public Sub GetMinExposureCompensation As Int
    r.target = parameters
    Return r.RunMethod("getMinExposureCompensation")
End Sub

'Public void setExposureCompensation (Int value)
'Sets the exposure compensation index.
'parameters value 	exposure compensation index.
'The valid value range Is from getMinExposureCompensation() (inclusive) To getMaxExposureCompensation() (inclusive).
'0 means exposure Is Not adjusted.
'Application should call getMinExposureCompensation AND getMaxExposureCompensation To know If exposure compensation Is supported.
Public Sub setExposureCompensation (expo As Int )
    r.target = parameters
    r.RunMethod2("setExposureCompensation", expo, "java.lang.int")
End Sub

'public boolean isAutoExposureLockSupported ()
Public Sub isAutoExposureLockSupported As Boolean
    r.target = parameters
    Return r.RunMethod("isAutoExposureLockSupported")
End Sub

'Public Boolean getAutoExposureLock ()
'Gets the state of the auto-exposure lock. Applications should check isAutoExposureLockSupported() before using this method.
Public Sub getAutoExposureLock As Boolean
    r.target = parameters
    Return r.RunMethod("getAutoExposureLock")
End Sub

'Public void setAutoExposureLock (Boolean toggle)
'Sets the auto-exposure lock state. Applications should check isAutoExposureLockSupported() before using this method.
'If set To True, the Camera auto-exposure routine will immediately pause Until the lock Is set To False. Exposure compensation settings changes will still take effect While auto-exposure Is locked.
'If auto-exposure Is already locked, setting this To True again has no effect (the driver will Not recalculate exposure values).
'Stopping preview with StopPreview(), OR triggering still image capture with TakePicture(Camera.ShutterCallback, Camera.PictureCallback, Camera.PictureCallback), will Not change the lock.
'Exposure compensation, auto-exposure lock, AND auto-white balance lock can be used To capture an exposure-bracketed burst of images, For example.
'Auto-exposure state, including the lock state, will Not be maintained after Camera Release() Is called. Locking auto-exposure after open() but before the first call To StartPreview() will Not allow the auto-exposure routine To run at all, AND may result In severely over- OR under-exposed images.
'parameters
'toggle 	new state of the auto-exposure lock. True means that auto-exposure Is locked, False means that the auto-exposure routine Is free To run normally.
Public Sub setAutoExposureLock (toogle As Boolean )
    r.target = parameters
    r.RunMethod2("setAutoExposureLock", toogle, "java.lang.boolean")
End Sub

'This method should only be called if you need to immediately release the camera.
'For example if you need to start another application that depends on the camera.
Public Sub CloseNow
	cam.Release
	r.target = cam
	r.RunMethod2("releaseCameras", True, "java.lang.boolean")
End Sub

'Calls AutoFocus and then takes the picture if focus was successfull.
Public Sub FocusAndTakePicture
	shoot = True
	cam.AutoFocus
End Sub

'Calls AutoFocus .
Public Sub Focus
	shoot = False
	cam.AutoFocus
End Sub

'Returns the camera rotation in degree (0, 90, 180, 270)
Public Sub GetRotation As Int
	r.target = r.GetActivity
	r.target = r.RunMethod("getWindowManager")
	r.target = r.RunMethod("getDefaultDisplay")
	r.target = r.RunMethod("getRotation")
	Dim degrees As Int = r.target * 90
	Dim ci As CameraInfoAndId = FindCamera(Front)
	r.target = ci.CameraInfo
	Dim orientation As Int = r.GetField("orientation")
	If Front Then
		Return (orientation + degrees) Mod 360
	Else
		Return (orientation - degrees + 360) Mod 360
	End If
End Sub

'Returns an array of CameraSize types containing the available preview sizes.
Public Sub GetSupportedPreviewSizes As CameraSize()
	r.target = parameters
	Dim list1 As List = r.RunMethod("getSupportedPreviewSizes")
	Dim cs(list1.Size) As CameraSize
	For i = 0 To list1.Size - 1
		r.target = list1.Get(i)
		cs(i).Width = r.GetField("width")
		cs(i).Height = r.GetField("height")
	Next
	Return cs
End Sub

Private Sub Camera_FocusDone (Success As Boolean)

	If Success Then
		If shoot Then TakePicture
		drawfocus(Colors.Green)
	
	Else
		Log("AutoFocus error.")
		'ToastMessageShow("unable to focus...", False)
		drawfocus(Colors.red)
	
		'TakePicture
	End If
End Sub

Private Sub drawfocus(focuscolor As Double)
	cFocus.Drawline( 0,0,0,pFocus.height ,focuscolor, 2dip)
	cFocus.DrawLine( 0, 0,pFocus.width, 0 ,focuscolor,2dip)
	cFocus.DrawLine( pFocus.width, pFocus.height,0, pFocus.height ,focuscolor,2dip)
	cFocus.DrawLine( pFocus.width, pFocus.height,pFocus.width, 0 ,focuscolor,2dip)
	pFocus.visible = False
	pFocus.visible =  True
End Sub